/// <reference types="node" />
import { ChildProcess } from 'child_process';
import { messages } from '@cucumber/messages';
import { EventEmitter } from 'events';
import { EventDataCollector } from '../../formatter/helpers';
import { IRuntimeOptions } from '..';
import { ISupportCodeLibrary } from '../../support_code_library_builder/types';
import { ICoordinatorReport, ICoordinatorReportSupportCodeIds } from './command_types';
export interface INewCoordinatorOptions {
    cwd: string;
    eventBroadcaster: EventEmitter;
    eventDataCollector: EventDataCollector;
    options: IRuntimeOptions;
    pickleIds: string[];
    supportCodeLibrary: ISupportCodeLibrary;
    supportCodePaths: string[];
    supportCodeRequiredModules: string[];
}
interface IWorker {
    closed: boolean;
    process: ChildProcess;
}
export default class Coordinator {
    private readonly cwd;
    private readonly eventBroadcaster;
    private readonly eventDataCollector;
    private readonly stopwatch;
    private onFinish;
    private nextPickleIdIndex;
    private readonly options;
    private readonly pickleIds;
    private workers;
    private supportCodeIdMap;
    private readonly supportCodeLibrary;
    private readonly supportCodePaths;
    private readonly supportCodeRequiredModules;
    private success;
    constructor({ cwd, eventBroadcaster, eventDataCollector, pickleIds, options, supportCodeLibrary, supportCodePaths, supportCodeRequiredModules, }: INewCoordinatorOptions);
    parseWorkerMessage(worker: IWorker, message: ICoordinatorReport): void;
    saveDefinitionIdMapping(message: ICoordinatorReportSupportCodeIds): void;
    remapDefinitionIds(testCase: messages.ITestCase): void;
    startWorker(id: string, total: number): void;
    onWorkerProcessClose(exitCode: number): void;
    parseTestCaseResult(testCaseFinished: messages.ITestCaseFinished): void;
    run(numberOfWorkers: number, done: (success: boolean) => void): void;
    giveWork(worker: IWorker): void;
    shouldCauseFailure(status: messages.TestStepFinished.TestStepResult.Status): boolean;
}
export {};
